@@ -4,7 +4,6 @@ from pydantic import BaseModel, Field
 from typing import List, Optional, Any, Union, Dict
 
 
-
 class RestrictionInfo(BaseModel):
     discountStatus:    Optional[int]
     ownerUid:          Optional[str]
@@ -170,6 +169,8 @@ class Auth(BaseModel):
     secret:   Optional[str]
     user:     Optional[UserProfile] = Field(alias="userProfile")
     deviceId: Optional[str]
+    
+    
 
 
 class Community(BaseModel):
@@ -537,7 +538,7 @@ class Thread(BaseModel):
     extensions:         Optional[Extensions]
     lastMessageSummary: Optional[LastMessage]
     tipInfo:            Optional[TipInfo]
-    userAddedTopicList: Optional[Topic]
+    # userAddedTopicList: Optional[Topic] Temporary
     uid:                Optional[str]
     membersSummary:     Optional[List[Member]]
     membersQuota:       Optional[int]
