@@ -65,6 +65,11 @@ class UserTypes:
     FEATURED: str = "featured"
     LEADERS: str =  "leaders"
     CURATORS: str = "curators"
+    
+class PostTypes:
+    BLOG: str = "blog"
+    WIKI: str = "wiki"
+    USER: str = "user"
 
 class LeadernoardTypes:
     DAY: int =        1
@@ -281,7 +286,11 @@ class PathTypes:
     EXTERNAL_CONTENT = "module.externalContent.enabled"
     TOPIC_CATEGORIES = "module.topicCategories.enabled"
     
-    LEFT_SIDE_PANEL_ICON_COLOR = "appearance.leftSidePanel.style.iconColor"    
+    LEFT_SIDE_PANEL_ICON_COLOR = "appearance.leftSidePanel.style.iconColor"   
+    
+class Language:
+    RU: str = "ru-UA"
+    ENG: str = "en-US" 
 
 class PromoteRankTypes:
     AGENT = "transfer-agent"
