@@ -1,24 +1,162 @@
 import json
 import sys
 
+from random import randint
 from uuid import uuid4
 from time import time
 from locale import localeconv
 from base64 import b64encode
-from typing import Dict, List, Literal, Optional, Union
+from typing import Dict, List, Optional, Union
 
 from aiohttp import BaseConnector, BasicAuth, ClientSession, ClientTimeout, ContentTypeError
 from json_minify import json_minify
 
 from .helpers.models import *
 
-from .helpers.types import GLOBAL_ID, ChatPublishTypes, ContentTypes, FeaturedTypes, ObjectTypes, PathTypes, RepairTypes, SourceTypes, UserTypes
-from .helpers.utils import generate_signature, generate_device, get_ndc, jsonify, decode_sid
+from .helpers.types import GLOBAL_ID, ChatPublishTypes, ContentTypes, FeaturedTypes, Language, ObjectTypes, PathTypes, PostTypes, RepairTypes, SourceTypes, UserTypes
+from .helpers.utils import generate_signature, generate_device, get_ndc, jsonify
 from .helpers.exceptions import CheckException, IpTomporaryBan, SpecifyType, HtmlError
 
 
+class WebHttpClient:
+    URL = "https://aminoapps.com/"
+    
+    def __init__(
+        self,
+        ndc_id: Optional[int] = None,
+        sid: Optional[str] = None,
+        session: Optional[ClientSession] = None,
+        connector: Optional[BaseConnector] = None,
+        debug: bool = False
+    ) -> None:
+        self.connector: Optional[BaseConnector] = connector
+        self.session: ClientSession = session or ClientSession()
+        self.debug = debug
+        self.debug = debug
+        
+        self.sid: Optional[str] = sid
+        self.ndc_id: int = ndc_id or GLOBAL_ID
+        
+        user_agent = "Amino.ed Python/{0[0]}.{0[1]} Bot"
+        self.user_agent: str = user_agent.format(sys.version_info)
+    
+    @property
+    def referer(self):
+        return f"{self.URL}/partial/main-chat-window?ndcId={self.ndc_id}"
+        
+    async def request(self, method: str, path: str, **kwargs):
+        url = f"{self.URL}api{path}"
+
+        headers: Dict[str, str] = {
+            "User-Agent": self.user_agent,
+            "x-requested-with": "xmlhttprequest",
+            "cookie": f"sid={self.sid}",
+            "referer": self.referer,
+            "host": "aminoapps.com"
+        }
+            
+        if kwargs.get("json") is not None:
+            headers["Content-Type"] = ContentTypes.JSON
+
+            data = kwargs.pop("json")
+            data["ndcId"] = get_ndc(self.ndc_id)[1:-2]
+
+            kwargs["data"] = json.dumps(data)
+        
+        kwargs["headers"] = headers
+        response_json: Optional[Dict] = None
+
+        async with self.session.request(method, url, **kwargs) as response:
+            try:
+                response_json: Dict = await response.json(loads=json.loads)
+            except ContentTypeError:
+                response_text = await response.text()
+                raise HtmlError(response_text)
+            
+            if self.debug:
+                message = f"\n\n<---REQUEST {url} START--->\n\n"
+                message += json.dumps(headers) + "\n"
+                
+                if data is not None:
+                    message += data + "\n"
+            
+                message += f"\n{response.status} {json.dumps(response_json)}\n\n"
+                message += f"<---REQUEST {url} END--->\n\n"
+                print(message, end="")
+
+            if response.status != 200:
+                if not self.session.closed:
+                    await self.session.close()
+
+                return CheckException(response_json)
+            
+            print(response.status, response_json)
+
+            return response_json or response.status
+
+    async def send_message(self, thread_id: str, message: str, type: int = 0):
+        data = jsonify(
+            threadId=thread_id,
+            message=jsonify(
+                content=message,
+                mediaType=0,
+                type=type,
+                sendFailed=False,
+                clientRefId=0
+            )
+        )
+
+        return await self.request("POST", f"/add-chat-message", json=data)
+    
+    # image - upload image link
+    async def send_image(self, thread_id: str, image: str):        
+        data = jsonify(
+            threadId=thread_id,
+            message=jsonify(
+                content=None,
+                mediaType=100,
+                type=0,
+                sendFailed=False,
+                clientRefId=0,
+                mediaValue=image
+            )
+        )
+
+        return await self.request("POST", f"/add-chat-message", json=data)
+    
+    async def join_chat(self, thread_id: str):
+        data = jsonify(threadId=thread_id)
+        return await self.request("POST", f"/join-thread", json=data)
+    
+    async def leave_chat(self, thread_id: str):
+        data = jsonify(threadId=thread_id)
+        return await self.request("POST", "/leave-thread", json=data)
+    
+    async def follow(self, uid: str):
+        data = jsonify(followee_id=uid)
+        return await self.request("POST", "/follow-user", json=data)
+    
+    async def start_chat(self, uids: list, message: str = None):
+        data = jsonify(inviteeUids=uids, initialMessageContent=message, type=0)
+        return await self.request("POST", "/create-chat-thread", json=data)
+    
+    async def comment_blog(self, blog_id: str, comment: str):
+        data = jsonify(content=comment, postType=PostTypes.BLOG, postId=blog_id)
+        return await self.request("POST", "/submit_comment", json=data)
+    
+    async def comment_user(self, uid: str, comment: str):
+        data = jsonify(content=comment, postType=PostTypes.USER, postId=uid)
+        return await self.request("POST", "/submit_comment", json=data)
+    
+    async def comment_wiki(self, wiki_id: str, comment: str):
+        data = jsonify(content=comment, postType=PostTypes.WIKI, postId=wiki_id)
+        return await self.request("POST", "/submit_comment", json=data)
+    
+
 class HttpClient:
-    API: str = "https://service.narvii.com/api/v1"
+    URL: str = "https://service.narvii.com/"
+    LANGUAGE: str = Language.ENG
+    _session: ClientSession = None
 
     def __init__(
         self,
@@ -27,10 +165,12 @@ class HttpClient:
         proxy: Optional[str] = None,
         proxy_auth: Optional[BasicAuth] = None,
         timeout: Optional[int] = None,
-        connector: Optional[BaseConnector] = None
+        connector: Optional[BaseConnector] = None,
+        debug: bool = False
     ) -> None:
         self.connector: Optional[BaseConnector] = connector
-        self._session: Optional[ClientSession] = session
+        self._session: Optional[ClientSession] = session or self._session
+        self.debug = debug
 
         self.ndc_id: int = GLOBAL_ID
         self._auth: Auth = Auth(**{})
@@ -44,19 +184,20 @@ class HttpClient:
         self.proxy: Optional[str] = proxy
         self.proxy_auth: Optional[BasicAuth] = proxy_auth
 
-        user_agent = "Amino.ed Python/{0[0]}.{0[1]} Bot"
-        self.user_agent: str = user_agent.format(sys.version_info)
+        user_agent = "Amino.ed Python/{0[0]}.{0[1]} Bot/{1}"
+        self.user_agent: str = user_agent.format(sys.version_info, randint(1, 9*9))
         
     async def request(self, method: str, path: str, **kwargs):
         ndc_id = kwargs.pop("ndc_id", self.ndc_id)
-        url = f"{self.API}{get_ndc(ndc_id)}{path}"
+        url = f"{self.URL}api/v1{get_ndc(ndc_id)}{path}"
             
         if kwargs.pop("full_url", None):
             url = kwargs.get("full_url", url)
 
         headers: Dict[str, str] = {
             "User-Agent": self.user_agent,
-            "NDCDEVICEID": self.device_id
+            "NDCDEVICEID": self.device_id,
+            "Accept-Language": self.LANGUAGE
         }
 
         if self.auth.sid is not None:
@@ -98,14 +239,39 @@ class HttpClient:
                     raise IpTomporaryBan("403 Forbidden")
                 else:
                     raise HtmlError(response_json)
+                
+            if self.debug:
+                message = f"\n\n<---REQUEST {url} START--->\n\n"
+                message += json.dumps(headers) + "\n"
+                
+                if data is not None:
+                    message += data + "\n"
+            
+                message += f"\n{response.status} {json.dumps(response_json)}\n\n"
+                message += f"<---REQUEST {url} END--->\n\n"
+                print(message, end="")
+                
+            if self.debug:
+                message = f"\n\n<---REQUEST {url} START--->\n\n"
+                message += json.dumps(headers) + "\n"
+                
+                if data is not None:
+                    message += data + "\n"
+            
+                message += f"\n{response.status} {json.dumps(response_json)}\n\n"
+                message += f"<---REQUEST {url} END--->\n\n"
+                print(message, end="")
 
             if response.status != 200:
                 if not self.session.closed:
                     await self.session.close()
 
                 return CheckException(response_json)
-
             return response_json or response.status
+    
+    @property
+    def web(self) -> WebHttpClient:
+        return WebHttpClient(self.ndc_id, self.auth.sid, self.session)
         
     @property
     def session(self) -> ClientSession:
@@ -141,43 +307,29 @@ class HttpClient:
     def auth(self, auth: Auth):
         self._auth = auth 
 
-    async def login_email(self, email: str, password: str, device_id: str = None) -> Auth:
+    async def base_login(
+        self, 
+        email: str = None, 
+        password: str = None, 
+        device_id: str = None, 
+        phoneNumber: str = None
+    ) -> Auth:
         data = jsonify(
             email=email,
             secret=password,
             clientType=100,
             deviceID=device_id or self.device_id,
-            action="normal"
-        )
-
-        response = Auth(**(await self.request("POST", "/auth/login", json=data, ndc_id=GLOBAL_ID)))
-        self.auth = response
-        self.auth.deviceId = self.device_id
-
-        return self.auth
-    
-    async def login_phone(self, phoneNumber: str, password: str, device_id: str = None) -> Auth:
-        data = jsonify(
-            phoneNumber=phoneNumber,
-            secret=password,
-            clientType=100,
-            deviceID=device_id or self.device_id,
-            action="normal"
-        )
-
-        response = Auth(**(await self.request("POST", "/auth/login", json=data, ndc_id=GLOBAL_ID)))
-        self.auth = response
-        self.auth.deviceId = self.device_id
-
-        return self.auth
-    
-    async def login_sid(self, sid: str) -> Auth:
-        self.auth = Auth(
-            sid=sid,
-            auid=decode_sid(sid).uid,
-            deviceId=self.device_id
+            action="normal",
+            v=2
         )
         
+        if phoneNumber:
+            data["phoneNumber"] = phoneNumber
+
+        response = Auth(**(await self.request("POST", "/auth/login", json=data, ndc_id=GLOBAL_ID)))
+        self.auth = response
+        self.auth.deviceId = self.device_id
+
         return self.auth
 
     async def logout(self, remove_auth: bool = True) -> Dict:
@@ -341,7 +493,7 @@ class HttpClient:
         return Community(**response["community"])
     
     async def get_account_communities(self, start: int = 0, size: int = 100) -> List[Community]:
-        response = await self.request("GET", f"/community/joined?v=1&start={start}&size={size}")
+        response = await self.request("GET", f"/community/joined?v=1&start={start}&size={size}", ndc_id=GLOBAL_ID)
         return list(map(lambda o: Community(**o), response["communityList"])) 
     
     async def search_community(self, amino_id: str):
@@ -1286,6 +1438,10 @@ class HttpClient:
         response = await self.request("GET", f"/feed/blog-all?{params}")
         return list(map(lambda o: Message(**o, **response.get("paging", {})),  response["blogList"]))
     
+    async def get_recent_wikis(self, start: int = 0, size: int = 25):
+        response = await self.request("GET", f"/item?type=catalog-all&start={start}&size={size}")
+        return list(map(lambda o: Wiki(**o), response["itemList"]))
+    
     async def get_notifications(self, start: int = 0, size: int = 100) -> Dict:
         return await self.request("GET", f"/notification?pagingType=t&start={start}&size={size}")["notificationList"]
     
@@ -1344,19 +1500,19 @@ class HttpClient:
         return await self.request("POST", f"/user-profile/featured/reorder", json=data)
     
     async def user_moderation_history(self, uid: str = None, size: int = 100) -> List[AdminLog]:
-        response = await self.request("GET", f"/admin/operation?pagingType=t&size={size}?objectId={uid}&objectType={ObjectTypes.USER}")
+        response = await self.request("GET", f"/admin/operation?pagingType=t&size={size}&objectId={uid}&objectType={ObjectTypes.USER}")
         return list(map(lambda o: AdminLog(**o), response["adminLogList"]))
 
     async def blog_moderation_history(self, blog_id: str = None, size: int = 100) -> List[AdminLog]:
-        response = await self.request("GET", f"/admin/operation?pagingType=t&size={size}?objectId={blog_id}&objectType={ObjectTypes.BLOG}")
+        response = await self.request("GET", f"/admin/operation?pagingType=t&size={size}&objectId={blog_id}&objectType={ObjectTypes.BLOG}")
         return list(map(lambda o: AdminLog(**o), response["adminLogList"]))
 
     async def wiki_moderation_history(self, wiki_id: str = None, size: int = 100) -> List[AdminLog]:
-        response = await self.request("GET", f"/admin/operation?pagingType=t&size={size}?objectId={wiki_id}&objectType={ObjectTypes.ITEM}")
+        response = await self.request("GET", f"/admin/operation?pagingType=t&size={size}&objectId={wiki_id}&objectType={ObjectTypes.ITEM}")
         return list(map(lambda o: AdminLog(**o), response["adminLogList"]))
 
     async def file_moderation_history(self, file_id: str = None, size: int = 100) -> List[AdminLog]:
-        response = await self.request("GET", f"/admin/operation?pagingType=t&size={size}?objectId={file_id}&objectType={ObjectTypes.FOLDER_FILE}")
+        response = await self.request("GET", f"/admin/operation?pagingType=t&size={size}&objectId={file_id}&objectType={ObjectTypes.FOLDER_FILE}")
         return list(map(lambda o: AdminLog(**o), response["adminLogList"]))
 
     async def feature_user(self, seconds: int, uid: str) -> int:
@@ -1578,7 +1734,7 @@ class HttpClient:
             )
         )
 
-        if self.auth.userProfile.membershipStatus == 0:
+        if self.auth.user.membershipStatus == 0:
             data["paymentContext"]["discountStatus"] = 0
 
         return await self.request("POST", f"/store/purchase", json=data)
@@ -1804,14 +1960,16 @@ class HttpClient:
         content: str = None, 
         aminoId: str = None, 
         primaryLanguage: str = None, 
-        themePackUrl: str = None
+        themePackUrl: str = None,
+        join_type: int = None # 0 - open, 1 - reqest, 2 - close
     ):
         data = jsonify(
             name=name,
             content=content,
             endpoint=aminoId,
             primaryLanguage=primaryLanguage,
-            themePackUrl=themePackUrl
+            themePackUrl=themePackUrl,
+            joinType=join_type
         )
 
         response = await self.request("POST", f"/community/settings", json=data)
