@@ -1,13 +1,15 @@
+import json
+import random
 import sys
 
 from time import time
 from contextlib import suppress
-from asyncio import AbstractEventLoop, sleep
+from asyncio import AbstractEventLoop, sleep, wait_for, exceptions
+from typing import Dict
 from aiohttp.client import ClientSession
 from aiohttp.client_exceptions import WSServerHandshakeError
 from aiohttp.client_ws import ClientWebSocketResponse as WSConnection
 from eventemitter.emitter import EventEmitter
-from ujson import loads
 
 from .helpers.models import Auth
 from .helpers.types import EventTypes, allTypes
@@ -27,6 +29,7 @@ class AminoWebSocket:
         self.reconnect_cooldown: int = 120
         
         self.bot_commands = []
+        self.wait_responses = {}
     
     async def run(self):
         self._connection = await self.create_connection()
@@ -34,6 +37,26 @@ class AminoWebSocket:
 
         self.reconnecting = True
         self._loop.create_task(self.reconnecting_task())
+        
+    async def send(self, data: Dict):
+        if "id" not in data:
+            data["id"] = "999345999"
+            
+        await self._connection.send_str(json.dumps(data))
+        
+    async def post(self, type: int, data: Dict):
+        data["id"] = str(random.randint(100000000,999999999))
+        self.wait_responses[data["id"]] = self._loop.create_future()
+        
+        await self.send({"o": data, "t": type})
+        
+        try:
+            response = self.wait_responses[data["id"]]
+            del self.wait_responses[data["id"]]
+            
+            return await wait_for(response, timeout=10)
+        except exceptions.TimeoutError:
+            return None
 
     async def connection_reciever(self):
         while True:
@@ -42,7 +65,7 @@ class AminoWebSocket:
                 continue
 
             with suppress(TypeError):
-                recieved_data = await self._connection.receive_json(loads=loads)
+                recieved_data = await self._connection.receive_json()
 
                 if recieved_data["t"] == 1000:
                     try:
@@ -80,6 +103,9 @@ class AminoWebSocket:
                 
                 self.emitter.emit(EventTypes.ANY, recieved_data)
                 
+                if recieved_data["o"].get("id") in self.wait_responses:
+                    self.wait_responses[recieved_data["o"].get("id")].set_result(recieved_data["o"])
+            
     async def create_connection(self) -> WSConnection:
         for _ in range(3):
             try:
@@ -94,7 +120,7 @@ class AminoWebSocket:
                 }
 
                 return await self._session.ws_connect(url, headers=headers)
-            except WSServerHandshakeError as e:
+            except WSServerHandshakeError:
                 await sleep(3)
                 await self._session.close()
         
